@@ -5,12 +5,14 @@ import { QueryPart } from 'app/core/components/query_part/query_part';
 import alertDef from './state/alertDef';
 import config from 'app/core/config';
 import appEvents from 'app/core/app_events';
-import { BackendSrv } from 'app/core/services/backend_srv';
+import { getBackendSrv } from '@grafana/runtime';
 import { DashboardSrv } from '../dashboard/services/DashboardSrv';
 import DatasourceSrv from '../plugins/datasource_srv';
-import { DataQuery } from '@grafana/ui/src/types/datasource';
+import { DataQuery, DataSourceApi, rangeUtil } from '@grafana/data';
 import { PanelModel } from 'app/features/dashboard/state';
 import { getDefaultCondition } from './getAlertingValidationMessage';
+import { CoreEvents } from 'app/types';
+import { promiseToDigest } from 'app/core/utils/promiseToDigest';
 
 export class AlertTabCtrl {
   panel: PanelModel;
@@ -30,14 +32,15 @@ export class AlertTabCtrl {
   appSubUrl: string;
   alertHistory: any;
   newAlertRuleTag: any;
+  alertingMinIntervalSecs: number;
+  alertingMinInterval: string;
+  frequencyWarning: any;
 
   /** @ngInject */
   constructor(
     private $scope: any,
-    private backendSrv: BackendSrv,
     private dashboardSrv: DashboardSrv,
     private uiSegmentSrv: any,
-    private $q: any,
     private datasourceSrv: DatasourceSrv
   ) {
     this.panelCtrl = $scope.ctrl;
@@ -51,6 +54,8 @@ export class AlertTabCtrl {
     this.executionErrorModes = alertDef.executionErrorModes;
     this.appSubUrl = config.appSubUrl;
     this.panelCtrl._enableAlert = this.enable;
+    this.alertingMinIntervalSecs = config.alertingMinInterval;
+    this.alertingMinInterval = rangeUtil.secondsToHms(config.alertingMinInterval);
   }
 
   $onInit() {
@@ -58,11 +63,11 @@ export class AlertTabCtrl {
 
     // subscribe to graph threshold handle changes
     const thresholdChangedEventHandler = this.graphThresholdChanged.bind(this);
-    this.panelCtrl.events.on('threshold-changed', thresholdChangedEventHandler);
+    this.panelCtrl.events.on(CoreEvents.thresholdChanged, thresholdChangedEventHandler);
 
     // set panel alert edit mode
     this.$scope.$on('$destroy', () => {
-      this.panelCtrl.events.off('threshold-changed', thresholdChangedEventHandler);
+      this.panelCtrl.events.off(CoreEvents.thresholdChanged, thresholdChangedEventHandler);
       this.panelCtrl.editingThresholds = false;
       this.panelCtrl.render();
     });
@@ -72,55 +77,61 @@ export class AlertTabCtrl {
     this.alertNotifications = [];
     this.alertHistory = [];
 
-    return this.backendSrv.get('/api/alert-notifications/lookup').then((res: any) => {
-      this.notifications = res;
+    return promiseToDigest(this.$scope)(
+      getBackendSrv()
+        .get('/api/alert-notifications/lookup')
+        .then((res: any) => {
+          this.notifications = res;
 
-      this.initModel();
-      this.validateModel();
-    });
+          this.initModel();
+          this.validateModel();
+        })
+    );
   }
 
   getAlertHistory() {
-    this.backendSrv
-      .get(`/api/annotations?dashboardId=${this.panelCtrl.dashboard.id}&panelId=${this.panel.id}&limit=50&type=alert`)
-      .then((res: any) => {
-        this.alertHistory = _.map(res, ah => {
-          ah.time = this.dashboardSrv.getCurrent().formatDate(ah.time, 'MMM D, YYYY HH:mm:ss');
-          ah.stateModel = alertDef.getStateDisplayModel(ah.newState);
-          ah.info = alertDef.getAlertAnnotationInfo(ah);
-          return ah;
-        });
-      });
+    promiseToDigest(this.$scope)(
+      getBackendSrv()
+        .get(`/api/annotations?dashboardId=${this.panelCtrl.dashboard.id}&panelId=${this.panel.id}&limit=50&type=alert`)
+        .then((res: any) => {
+          this.alertHistory = _.map(res, ah => {
+            ah.time = this.dashboardSrv.getCurrent().formatDate(ah.time, 'MMM D, YYYY HH:mm:ss');
+            ah.stateModel = alertDef.getStateDisplayModel(ah.newState);
+            ah.info = alertDef.getAlertAnnotationInfo(ah);
+            return ah;
+          });
+        })
+    );
   }
 
   getNotificationIcon(type: string): string {
     switch (type) {
       case 'email':
-        return 'fa fa-envelope';
+        return 'envelope';
       case 'slack':
-        return 'fa fa-slack';
+        return 'slack';
       case 'victorops':
         return 'fa fa-pagelines';
       case 'webhook':
-        return 'fa fa-cubes';
+        return 'cube';
       case 'pagerduty':
         return 'fa fa-bullhorn';
       case 'opsgenie':
-        return 'fa fa-bell';
+        return 'bell';
       case 'hipchat':
         return 'fa fa-mail-forward';
       case 'pushover':
-        return 'fa fa-mobile';
+        return 'mobile-android';
       case 'kafka':
-        return 'fa fa-random';
+        return 'arrow-random';
       case 'teams':
         return 'fa fa-windows';
     }
-    return 'fa fa-bell';
+    return 'bell';
   }
 
   getNotifications() {
-    return this.$q.when(
+    return Promise.resolve(
       this.notifications.map((item: any) => {
         return this.uiSegmentSrv.newSegment(item.name);
       })
@@ -156,8 +167,8 @@ export class AlertTabCtrl {
   removeNotification(an: any) {
     // remove notifiers refeered to by id and uid to support notifiers added
     // before and after we added support for uid
-    _.remove(this.alert.notifications, (n: any) => n.uid === an.uid || n.id === an.id);
-    _.remove(this.alertNotifications, (n: any) => n.uid === an.uid || n.id === an.id);
+    _.remove(this.alert.notifications, (n: any) => n.uid === an.uid);
+    _.remove(this.alertNotifications, (n: any) => n.uid === an.uid);
   }
 
   addAlertRuleTag() {
@@ -178,6 +189,8 @@ export class AlertTabCtrl {
       return;
     }
 
+    this.checkFrequency();
+
     alert.conditions = alert.conditions || [];
     if (alert.conditions.length === 0) {
       alert.conditions.push(getDefaultCondition());
@@ -200,7 +213,7 @@ export class AlertTabCtrl {
         memo.push(this.buildConditionModel(value));
         return memo;
       },
-      []
+      [] as string[]
     );
 
     ThresholdMapper.alertToGraphThresholds(this.panel);
@@ -212,6 +225,19 @@ export class AlertTabCtrl {
       // fallback to using id if uid is missing
       if (!model) {
         model = _.find(this.notifications, { id: addedNotification.id });
+        if (!model) {
+          appEvents.emit(CoreEvents.showConfirmModal, {
+            title: 'Notifier with invalid ID is detected',
+            text: `Do you want to delete notifier with invalid ID: ${addedNotification.id} from the dashboard JSON?`,
+            text2: 'After successful deletion, make sure to save the dashboard for storing the update JSON.',
+            icon: 'trash-alt',
+            confirmText: 'Delete',
+            yesText: 'Delete',
+            onConfirm: async () => {
+              this.removeNotification(addedNotification);
+            },
+          });
+        }
       }
 
       if (model && model.isDefault === false) {
@@ -223,7 +249,6 @@ export class AlertTabCtrl {
     for (const notification of this.notifications) {
       if (notification.isDefault) {
         notification.iconClass = this.getNotificationIcon(notification.type);
-        notification.bgColor = '#00678b';
         this.alertNotifications.push(notification);
       }
     }
@@ -232,6 +257,27 @@ export class AlertTabCtrl {
     this.panelCtrl.render();
   }
 
+  checkFrequency() {
+    if (!this.alert.frequency) {
+      return;
+    }
+
+    this.frequencyWarning = '';
+
+    try {
+      const frequencySecs = rangeUtil.intervalToSeconds(this.alert.frequency);
+      if (frequencySecs < this.alertingMinIntervalSecs) {
+        this.frequencyWarning =
+          'A minimum evaluation interval of ' +
+          this.alertingMinInterval +
+          ' have been configured in Grafana and will be used for this alert rule. ' +
+          'Please contact the administrator to configure a lower interval.';
+      }
+    } catch (err) {
+      this.frequencyWarning = err;
+    }
+  }
+
   graphThresholdChanged(evt: any) {
     for (const condition of this.alert.conditions) {
       if (condition.type === 'query') {
@@ -248,8 +294,9 @@ export class AlertTabCtrl {
     }
 
     let firstTarget;
-    let foundTarget: DataQuery = null;
+    let foundTarget: DataQuery | null = null;
 
+    const promises: Array<Promise<any>> = [];
     for (const condition of this.alert.conditions) {
       if (condition.type !== 'query') {
         continue;
@@ -271,20 +318,34 @@ export class AlertTabCtrl {
           foundTarget = firstTarget;
         } else {
           this.error = 'Could not find any metric queries';
+          return;
         }
       }
 
       const datasourceName = foundTarget.datasource || this.panel.datasource;
-      this.datasourceSrv.get(datasourceName).then(ds => {
-        if (!ds.meta.alerting) {
-          this.error = 'The datasource does not support alerting queries';
-        } else if (ds.targetContainsTemplate && ds.targetContainsTemplate(foundTarget)) {
-          this.error = 'Template variables are not supported in alert queries';
-        } else {
-          this.error = '';
-        }
-      });
+      promises.push(
+        this.datasourceSrv.get(datasourceName).then(
+          (foundTarget => (ds: DataSourceApi) => {
+            if (!ds.meta.alerting) {
+              return Promise.reject('The datasource does not support alerting queries');
+            } else if (ds.targetContainsTemplate && ds.targetContainsTemplate(foundTarget)) {
+              return Promise.reject('Template variables are not supported in alert queries');
+            }
+            return Promise.resolve();
+          })(foundTarget)
+        )
+      );
     }
+    Promise.all(promises).then(
+      () => {
+        this.error = '';
+        this.$scope.$apply();
+      },
+      e => {
+        this.error = e;
+        this.$scope.$apply();
+      }
+    );
   }
 
   buildConditionModel(source: any) {
@@ -304,7 +365,7 @@ export class AlertTabCtrl {
         break;
       }
       case 'get-part-actions': {
-        return this.$q.when([]);
+        return Promise.resolve([]);
       }
       case 'part-param-changed': {
         this.validateModel();
@@ -314,9 +375,14 @@ export class AlertTabCtrl {
           return this.uiSegmentSrv.newSegment({ value: target.refId });
         });
 
-        return this.$q.when(result);
+        return Promise.resolve(result);
+      }
+      default: {
+        return Promise.resolve();
       }
     }
+
+    return Promise.resolve();
   }
 
   handleReducerPartEvent(conditionModel: any, evt: any) {
@@ -333,9 +399,11 @@ export class AlertTabCtrl {
             result.push(type);
           }
         }
-        return this.$q.when(result);
+        return Promise.resolve(result);
       }
     }
+
+    return Promise.resolve();
   }
 
   addCondition(type: string) {
@@ -352,11 +420,11 @@ export class AlertTabCtrl {
   }
 
   delete() {
-    appEvents.emit('confirm-modal', {
+    appEvents.emit(CoreEvents.showConfirmModal, {
       title: 'Delete Alert',
       text: 'Are you sure you want to delete this alert rule?',
       text2: 'You need to save dashboard for the delete to take effect',
-      icon: 'fa-trash',
+      icon: 'trash-alt',
       yesText: 'Delete',
       onConfirm: () => {
         delete this.panel.alert;
@@ -402,21 +470,23 @@ export class AlertTabCtrl {
   }
 
   clearHistory() {
-    appEvents.emit('confirm-modal', {
+    appEvents.emit(CoreEvents.showConfirmModal, {
       title: 'Delete Alert History',
       text: 'Are you sure you want to remove all history & annotations for this alert?',
-      icon: 'fa-trash',
+      icon: 'trash-alt',
       yesText: 'Yes',
       onConfirm: () => {
-        this.backendSrv
-          .post('/api/annotations/mass-delete', {
-            dashboardId: this.panelCtrl.dashboard.id,
-            panelId: this.panel.id,
-          })
-          .then(() => {
-            this.alertHistory = [];
-            this.panelCtrl.refresh();
-          });
+        promiseToDigest(this.$scope)(
+          getBackendSrv()
+            .post('/api/annotations/mass-delete', {
+              dashboardId: this.panelCtrl.dashboard.id,
+              panelId: this.panel.id,
+            })
+            .then(() => {
+              this.alertHistory = [];
+              this.panelCtrl.refresh();
+            })
+        );
       },
     });
   }
